<?php
// repository-manager

$logger = true;

if (version_compare(PHP_VERSION, '5', '>='))
  require_once('domxml-php4-to-php5.php'); //Load the PHP5 converter

error_reporting(E_NONE);

define("XML_BADIP", "Connection denied");
define("XML_BADPOST", "error parsing POST data");
define("XML_BADROOT", "Invalid request");
define("XML_INVALIDSESSION", "Invalid session info");
define("XML_UNKNOWN", "Unknown error while processing your request");
define("XML_UNSUPPORTED", "Unsupported method");

require_once("parameters_inc.php");
require_once("oai_functions.php");
require_once("oaicommon.php");
require_once("oaidb.php");
require_once("access.php");

if (!ConnectionAllowed($_SERVER["REMOTE_ADDR"], $allowManagerConnect)) {
  XMLError(XML_BADIP);
}

#error_reporting(E_ALL);

function logger($data) {
  global $logger;

  if ($logger) {
    $fp = fopen(sprintf("manager-%s.log", gmdate("Y-m-d", time())), 'a');
    foreach(explode("\n", $data) as $zendfix => $val) {
      if ($val !== "") {
        fwrite($fp, sprintf("%s %s\n", ISO8601Date(), $val));
      }
    }
    fclose($fp);
  }
  return $data;
}

function XMLHeader() {
  global $XMLHeaderSent;

  if ($XMLHeaderSent != 1) {
    $XMLHeaderSent = 1;
    header("Content-Type: text/xml;charset=utf-8");
    print "<" . "?xml version = \"1.0\" encoding = \"utf-8\"?" . ">\n";
  }
}

function XMLError($errstr, $rootitem = "error") {
  global $logger;

  XMLHeader();
  print XMLtag($rootitem, array("errorCode" => 1, "errorMessage" => $errstr));

  if ($logger) {
    ob_end_flush();
  }

  exit;
}

function ReadInput() {
  $postdata = fopen("php://input", "r");
  $data = "";
  while ($tmpdata = fread($postdata, 4096))
    $data .= $tmpdata;
  fclose($postdata);
  return $data;
}

function StartSession($domroot) {
  global $manager;

  session_id(getSubelementContent($domroot, "sessionID"));
  session_start();
  if ((!array_key_exists($_SESSION["user"], $manager)) || ($manager[$_SESSION["user"]] !== $_SESSION["password"])) {
    XMLError(XML_INVALIDSESSION);
  }
}

function SetMinMax($set, &$min, &$max) {
  if (strcasecmp($set, $max) > 0) {
    $max = $set;
  }
  if (strcasecmp($set, $min) < 0) {
    $min = $set;
  }
}

function IsInRange($what, $in_array) {
  if ((!array_key_exists("start", $in_array) || (strcmp($in_array["start"], $what) <= 0)) &&
      (!array_key_exists("end",   $in_array) || (strcmp($in_array["end"],   $what) >= 0))) {
    return TRUE;
  } else {
    return FALSE;
  }
}

function IsInRangeNum($what, $in_array) {
  if ((!array_key_exists("start", $in_array) || ($in_array["start"] <= $what)) &&
      (!array_key_exists("end",   $in_array) || ($in_array["end"]   >  $what))) {
    return TRUE;
  } else {
    return FALSE;
  }
}

function RulesetFileInfo() {
  if (is_readable("MetaEditorRuleset.xml")) {
    $fp = fopen("MetaEditorRuleset.xml", "r");
    $fstat = fstat($fp);
    fclose($fp);
    return XMLtag("rulesetFile", NULL, ISO8601Date(max($fstat["mtime"], $fstat["ctime"])));
  } else {
    return "";
  }
}

function ManagerLogin($domroot) {
  global $manager;

  $domuser = getSubelementContent($domroot, "user");
  $dompassword = getSubelementContent($domroot, "password");
  if ($manager[$domuser] !== $dompassword) {
    XMLHeader();
    print XMLtag("login", NULL, XMLtag("result", NULL, 0));
  } else {
    session_start();
    $_SESSION["user"] = $domuser;
    $_SESSION["password"] = $dompassword;
    #session_write_close();
    XMLHeader();
    print XMLtag("login", NULL, XMLtag("result", NULL, 1) . XMLtag("sessionID", NULL, session_id()) . 
      RulesetFileInfo());
  }
}

function ManagerListSchemas($domroot) {
  global $repository, $config, $revmetadataFormats;

  StartSession($domroot);
  XMLHeader();
  $records = getRecordsNew($repository);
  $data = array();
  foreach($revmetadataFormats as $key => $value) {
    $data[$key] = array();
    $data[$key]["root_name"] = $key;
    $data[$key]["createdmin"] = "9999-12-31T23:59:59Z";
    $data[$key]["createdmax"] = "0001-01-01T00:00:00Z";
    $data[$key]["modifiedmin"] = "9999-12-31T23:59:59Z";
    $data[$key]["modifiedmax"] = "0001-01-01T00:00:00Z";
    $data[$key]["deletedmin"] = "9999-12-31T23:59:59Z";
    $data[$key]["deletedmax"] = "0001-01-01T00:00:00Z";
    $data[$key]["recordCount"] = 0;
    $data[$key]["deletedRecordCount"] = 0;
  }
  foreach($records as $id => $value) {
    foreach($value as $root_name => $onerecord) {
      if ($onerecord["created"] != $onerecord["deleted"]) {
        $data[$root_name]["deletedRecordCount"]++;
      } else {
        $data[$root_name]["recordCount"]++;
      }
      SetMinMax($onerecord["created"], $data[$root_name]["createdmin"], $data[$root_name]["createdmax"]);
      SetMinMax($onerecord["modified"], $data[$root_name]["modifiedmin"], $data[$root_name]["modifiedmax"]);
      SetMinMax($onerecord["deleted"], $data[$root_name]["deletedmin"], $data[$root_name]["deletedmax"]);
    }
  }

  print XMLtagStart("listSchemas", array("errorCode" => 0));
  foreach($data as $key => $value) {
    print XMLtagStart("schema");

    print XMLtag("recordCount", NULL, $value["recordCount"]);
    print XMLtag("deletedRecordCount", NULL, $value["deletedRecordCount"]);
    print XMLtag("type", NULL, $revmetadataFormats[$key]["rootElementName"]);
    print XMLtag("namespaceURI", NULL, $revmetadataFormats[$key]["namespace"]);
    print XMLtag("schemaLocation", NULL, $revmetadataFormats[$key]["schema"]);
    if (($value["recordCount"] != 0) || ($value["deletedRecordCount"] != 0)) {
      print XMLtag("creationInterval", array("start" => $value["createdmin"], "end" => $value["createdmax"]));
      print XMLtag("modificationInterval", array("start" => $value["modifiedmin"], "end" => $value["modifiedmax"]));
      print XMLtag("deletionInterval", array("start" => $value["deletedmin"], "end" => $value["deletedmax"]));
    } else {
      print XMLtag("creationInterval");
      print XMLtag("modificationInterval");
      print XMLtag("deletionInterval");
    }

    print XMLtagEnd("schema");
  }

  print XMLtagEnd("listSchemas");
}

function ManagerGetRecords($domroot) {
  global $repository, $config;

  StartSession($domroot);
  XMLHeader();

  $domfilter = $domroot->get_elements_by_tagname("filter");
  if (count($domfilter) == 0) {
    XMLError(XML_BADROOT);
  }
  $domfilterarray = $domfilter[0]->child_nodes();
  $filter = array();
  $filter["excludeType"] = array();
  $filter["identifier"] = array();
  $filter["created"] = array();
  $filter["modified"] = array();
  $filter["deleted"] = array();
  $filter["rows"] = array();
  $filter["needDeleted"] = FALSE;
  foreach($domfilterarray as $zendfix => $value) {
    switch ($value->tagname) {
      case "excludeType":
        $filter["excludeType"][$value->get_content()] = 1;
        break;
      case "identifier":
        $filter["identifier"]["start"] = $value->get_attribute("start");
        $filter["identifier"]["end"] = $value->get_attribute("end");
        break;
      case "creation":
        $filter["created"]["start"] = $value->get_attribute("start");
        $filter["created"]["end"] = $value->get_attribute("end");
        break;
      case "modification":
        $filter["modified"]["start"] = $value->get_attribute("start");
        $filter["modified"]["end"] = $value->get_attribute("end");
        break;
      case "deletion":
        $filter["deleted"]["start"] = $value->get_attribute("start");
        $filter["deleted"]["end"] = $value->get_attribute("end");
        $filter["needDeleted"] = TRUE;
        break;
      case "rows":
        $filter["rows"]["start"] = $value->get_attribute("start");
        $filter["rows"]["end"] = $value->get_attribute("end");
        break;
    }
  }

  $records = getRecordsNew($repository);
  $recordCount = 0;
  foreach($records as $id => $value) {
    foreach($value as $root_name => $onerecord) {
      $oai_id = "oai:" . $config["repositoryId"] . ":" . $onerecord["id"];
      if (!array_key_exists($root_name, $filter["excludeType"])) {
        if (IsInRange($oai_id, $filter["identifier"]) &&
            IsInRange($onerecord["created"], $filter["created"]) &&
            IsInRange($onerecord["modified"], $filter["modified"]) &&
            IsInRange($onerecord["deleted"], $filter["deleted"]) &&
            (($onerecord["deleted"] == $onerecord["created"]) || $filter["needDeleted"]) ) {
          $recordCount++;
        }
      }
    }
  }
  # copy-paste programming - yay \o/
  print XMLtagStart("getRecords", array("recordCount" => $recordCount, "errorCode" => 0));
  $rownum = 0;
  foreach($records as $id => $value) {
    foreach($value as $root_name => $onerecord) {
      $oai_id = "oai:" . $config["repositoryId"] . ":" . $onerecord["id"];
      if (!array_key_exists($root_name, $filter["excludeType"])) {
        if (IsInRange($oai_id, $filter["identifier"]) &&
            IsInRange($onerecord["created"], $filter["created"]) &&
            IsInRange($onerecord["modified"], $filter["modified"]) &&
            IsInRange($onerecord["deleted"], $filter["deleted"]) &&
            (($onerecord["deleted"] == $onerecord["created"]) || $filter["needDeleted"]) ) {
          if (IsInRangeNum($rownum, $filter["rows"])) {
            print XMLtagStart("record");
            print XMLtag("identifier", NULL, $oai_id);
            print XMLtag("creationDate", NULL, $onerecord["created"]);
            print XMLtag("modificationDate", NULL, $onerecord["modified"]);
            if (strcmp($onerecord["created"], $onerecord["deleted"]) != 0) {
              print XMLtag("deletionDate", NULL, $onerecord["deleted"]);
            } else {
              print XMLtag("deletionDate");
            }
            print XMLtag("content", NULL, $onerecord["xml"]);
            print XMLtagEnd("record");
          }
          $rownum++;
        }
      }
    }
  }
  print XMLtagEnd("getRecords");
}

# TODO:HACK: nagy hack, ki kell cserelni egy _jo_ eljarasra!
function MakeXML($work, $num = -1) {
  $b = $work->owner_document();
  $c = $b->dump_mem(TRUE, "utf-8");
  if ($num == -1) {
    # ha identifier alapjan keresunk
    if (preg_match("/<identifier>" . getSubelementContent($work, "identifier") . "<\/identifier>[^<]*?<content>(.*?)<\/content>/ms", $c, $matches)) {
      return $matches[1];
    }
  } else {
    # ha sorszam alapjan keresunk
    #if (preg_match_all("/<(modify|delete|create)>.*?<content>(.*?)<\/content>/ms", $c, $matches, PREG_SET_ORDER)) {
    #   return $matches[$num][2];
    #}
    if (preg_match_all("#<(modify|delete|create)>(.*?)</(modify|delete|create)>#ms", $c, $matches, PREG_SET_ORDER)) {
      $tmp = $matches[$num][2];
      if (preg_match("#<content>(.*?)</content>#ms", $tmp, $matches)) {
        return $matches[1];
      } else {
        return FALSE;
      }
    }
  }
  return FALSE;
}

function getRootName() {
	
}

function ManagerCommitCreate(&$db, $work, $NR, &$schemas) {
  global $repository, $metadataFormats, $maxid;

  $root_namespace = getSubelementContent($work, "namespaceURI");
  $root_name = NULL;
  foreach ($metadataFormats as $format_name => $value) {
    if (strcmp($root_namespace, $value["namespace"]) == 0) {
      $root_name = $value["rootElementName"];
    }
  }
  if ($root_name !== NULL) {
	logger("create row: $root_name");
    $schemas[$root_name] = true;
    $data = MakeXML($work, $NR);
    if ($data === FALSE) {
      return 0;
    }
    $maxid++;
    $mydate = ISO8601Date();
    $result = WriteRecord($db, $repository["table"], array("id" => $maxid, "sett" => "",
      "created" => $mydate, "modified" => $mydate, "deleted" => $mydate,
      "xml" => $data, "harvestable" => ynHarvestable($data),
      "root_name" => $root_name, "root_namespace" => $root_namespace));
    $db->commit();
    return ($result ? 1 : 0);
  } else {
    return 0;
  }
}

function ManagerCommitModify(&$db, $work, &$schemas) {
  global $repository;
  $data = MakeXML($work);
  if ($data === FALSE) {
    return 0;
  }

  if(preg_match("/^\\s*<[^:]+:([^\\s]+)/i", $data, $r)) {
    $schemas[$r[1]] = true;
  }

  $result = $db->update($repository["table"], array("modified" => $db->escapedate(ISO8601Date()), "xml" => $db->escapestr($data), "harvestable" => $db->escapestr(ynHarvestable($data))), "id = " . $db->escapestr(ExplodeID(getSubelementContent($work, "identifier"))));

  return ($result ? 1 : 0);
}

function ManagerCommitDelete(&$db, $work, &$schemas) {
  global $repository, $config;

  if ($config["deletedRecord"] === "no") {
    $result = $db->delete($repository["table"], "id = " . $db->escapestr(ExplodeID(getSubelementContent($work, "identifier"))));
  } else {
    $result = $db->update($repository["table"], array("deleted" => $db->escapedate(ISO8601Date())), "id = " . $db->escapestr(ExplodeID(getSubelementContent($work, "identifier"))));
  }

  return ($result ? 1 : 0);
}

function ManagerCommitChanges($domroot) {
  global $repository, $maxid;

  StartSession($domroot);
  XMLHeader();

  $domchangearray = $domroot->child_nodes();
  $EC = 0;
  $NR = 0;
  $schemas = Array();

  $db = new transdb($repository["type"], $repository["host"], $repository["user"], $repository["password"], $repository["database"]);
  $db->ErrorReporting($repository["warn"]);
  getRecordsNew($repository, FALSE, FALSE);
  foreach($domchangearray as $zendfix => $work) {
    switch ($work->tagname) {
      case "create":
        $EC += ManagerCommitCreate($db, $work, $NR, $schemas);
        $NR++;
        break;
      case "modify":
        $EC += ManagerCommitModify($db, $work, $schemas);
        $NR++;
        break;
      case "delete":
        $EC += ManagerCommitDelete($db, $work, $schemas);
        $NR++;
        break;
    }
  }
  $db->SQL_disconnect();
  triggerHarvester(array_keys($schemas));
  if ($NR != $EC) {
    XMLError($NR . "/" . $EC . " error(s)", "commitChanges");
  } else {
    print XMLTag("commitChanges", array("errorCode" => 0));
  }
}

function ManagerGetRuleset($domroot) {
  global $repository, $config, $revmetadataFormats;

  StartSession($domroot);
  XMLHeader();

  if (is_readable("MetaEditorRuleset.xml")) {
    print XMLtagStart("getRuleset", array("errorCode" => 0));
    $fp = fopen("MetaEditorRuleset.xml", "r");
    fpassthru($fp);
    fclose($fp);
    print XMLtagEnd("getRuleset");
  } else {
    XMLError("Cannot open file", "getRuleset");
  }
}

function main() {
  global $logger;

  if ($logger) {
    ob_start("logger");
  }
  if ($_SERVER["REQUEST_METHOD"] !== "POST") {
    XMLError(XML_UNSUPPORTED);
  } else {
    $input = ReadInput();
    logger("### INPUT ###");
    logger($input);
    logger("### OUTPUT ###");
    if (! $dom = domxml_open_mem($input, DOMXML_LOAD_DONT_KEEP_BLANKS | DOMXML_LOAD_PARSING)) {
      XMLError(XML_BADPOST);
    } else {
      $domroot = $dom->document_element();
      switch ($domroot->tagname) {
        case "login":
          ManagerLogin($domroot);
          break;
        case "listSchemas":
          ManagerListSchemas($domroot);
          break;
        case "getRecords":
          ManagerGetRecords($domroot);
          break;
        case "commitChanges":
          ManagerCommitChanges($domroot);
          break;
        case "getRuleset":
          ManagerGetRuleset($domroot);
          break;
        default :
          XMLError(XML_BADROOT);
      }
    }
  }
  if ($logger) {
    ob_end_flush();
  }
}

main();

?>
